@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import typing
+from gettext import gettext as _
 from gi.repository import Adw, Gtk
 
 if typing.TYPE_CHECKING:
@@ -13,14 +14,11 @@ class AttributeEntryRow(Adw.EntryRow):
 
     __gtype_name__ = "AttributeEntryRow"
 
-    def __init__(
-        self, entry: SafeEntry, key: str, value: str, list_box: Gtk.ListBox
-    ) -> None:
+    def __init__(self, entry: SafeEntry, key: str, value: str) -> None:
         super().__init__()
 
         self.entry = entry
         self.key = key
-        self.list_box = list_box
 
         self.props.title = key
         if value:
@@ -28,8 +26,18 @@ class AttributeEntryRow(Adw.EntryRow):
 
     @Gtk.Template.Callback()
     def _on_remove_button_clicked(self, _button):
+        window = self.get_root()
         self.entry.delete_attribute(self.key)
-        self.list_box.remove(self)
+        window.unlocked_db.attribute_deleted(
+            self.entry, self.key, self.props.text, False
+        )
+
+    @Gtk.Template.Callback()
+    def _on_copy_button_clicked(self, _row):
+        clipboard = self.get_clipboard()
+        clipboard.set(self.props.text)
+        window = self.get_root()
+        window.send_notification(_("Attribute copied"))
 
     @Gtk.Template.Callback()
     def _on_changed(self, row):
