@@ -2,7 +2,6 @@
 from __future__ import annotations
 
 import logging
-import threading
 from enum import IntEnum
 from gettext import gettext as _
 
@@ -14,6 +13,7 @@ from gsecrets.database_manager import DatabaseManager
 from gsecrets.save_dialog import SaveDialog
 from gsecrets.settings_dialog import SettingsDialog
 from gsecrets.unlock_database import UnlockDatabase
+from gsecrets.widgets.quit_conflict_dialog import QuitConflictDialog
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/window.ui")
@@ -22,24 +22,20 @@ class Window(Adw.ApplicationWindow):
     # pylint: disable=too-many-public-methods
     class View(IntEnum):
         WELCOME = 0
-        RECENT_FILES = 1
         UNLOCK_DATABASE = 2
         UNLOCKED_DATABASE = 3
         CREATE_DATABASE = 4
 
     __gtype_name__ = "Window"
 
+    _create_view = None
     unlocked_db = None
 
     _create_database_bin = Gtk.Template.Child()
-    create_database_headerbar = Gtk.Template.Child()
     _main_view = Gtk.Template.Child()
-    _recent_files_page = Gtk.Template.Child()
     _spinner = Gtk.Template.Child()
-    _headerbar_stack = Gtk.Template.Child()
-    _toast_overlay = Gtk.Template.Child()
+    toast_overlay = Gtk.Template.Child()
     _unlock_database_bin = Gtk.Template.Child()
-    unlock_database_headerbar = Gtk.Template.Child()
     unlocked_db_bin = Gtk.Template.Child()
 
     mobile_layout = GObject.Property(type=bool, default=False)
@@ -58,46 +54,23 @@ class Window(Adw.ApplicationWindow):
 
     def send_notification(self, notification: str) -> None:
         toast = Adw.Toast.new(notification)
-        self._toast_overlay.add_toast(toast)
-
-    def set_headerbar(self, headerbar: Adw.HeaderBar) -> None:
-        self.add_headerbar(headerbar)
-        self._headerbar_stack.set_visible_child(headerbar)
-
-    def add_headerbar(self, headerbar: Adw.HeaderBar) -> None:
-        if not self._headerbar_stack.get_page(headerbar):
-            self._headerbar_stack.add_child(headerbar)
+        self.toast_overlay.add_toast(toast)
 
     def assemble_window(self) -> None:
         window_size = gsecrets.config_manager.get_window_size()
         self.set_default_size(window_size[0], window_size[1])
 
-        self.apply_theme()
-
-        if Gio.Application.get_default().development_mode:
+        if self.props.application.development_mode:
             self.add_css_class("devel")
 
-    def apply_theme(self) -> None:
-        """Set the bright/dark theme depending on the configuration"""
-        manager = Adw.StyleManager.get_default()
-        if not manager.props.system_supports_color_schemes:
-            dark_theme = gsecrets.config_manager.get_dark_theme()
-            if dark_theme:
-                manager.props.color_scheme = Adw.ColorScheme.PREFER_DARK
-            else:
-                manager.props.color_scheme = Adw.ColorScheme.DEFAULT
+    def do_enable_debugging(  # pylint: disable=arguments-differ
+        self, toggle: bool
+    ) -> bool:
+        if not self.application.development_mode:
+            logging.warning("The inspector is not enabled in non-development builds")
+            return False
 
-    def do_size_allocate(
-        self, width: int, height: int, baseline: int
-    ) -> None:  # pylint: disable=arguments-differ
-        # pylint: disable=arguments-differ
-        """Handler for resizing event. It is used to check if
-        the layout needs to be updated."""
-        new_mobile_layout = width < 700
-        if new_mobile_layout != self.props.mobile_layout:
-            self.props.mobile_layout = new_mobile_layout
-
-        Adw.ApplicationWindow.do_size_allocate(self, width, height, baseline)
+        return Adw.ApplicationWindow.do_enable_debugging(self, toggle)
 
     def invoke_initial_screen(self) -> None:
         """Present the first start screen if required or autoload files
@@ -114,7 +87,7 @@ class Window(Adw.ApplicationWindow):
             gfile: Gio.File = Gio.File.new_for_uri(
                 gsecrets.config_manager.get_last_opened_database()
             )
-            if Gio.File.query_exists(gfile):
+            if gfile.query_exists():
                 filepath = gfile.get_path()
                 logging.debug("Opening last opened database: %s", filepath)
                 self.start_database_opening_routine(filepath)
@@ -123,80 +96,63 @@ class Window(Adw.ApplicationWindow):
         # Display the screen with last opened files (or welcome page)
         if not gsecrets.config_manager.get_last_opened_list():
             logging.debug("No recent files saved")
-            self.view = self.View.WELCOME
-            return
 
-        if self._recent_files_page.is_empty:
-            logging.debug("No recent files")
-            self.view = self.View.WELCOME
-            return
-
-        self.view = self.View.RECENT_FILES
+        self.view = self.View.WELCOME
 
     #
     # Open Database Methods
     #
 
-    def on_open_database_action(
-        self, _action: Gio.Action, _param: GLib.Variant
-    ) -> None:
+    def on_open_database_action(self, _action: Gio.Action, param: GLib.Variant) -> None:
         """Callback function to open a safe."""
         # pylint: disable=too-many-locals
-        opening_dialog = Gtk.FileChooserNative.new(
-            # NOTE: Filechooser title for opening an existing keepass safe kdbx file
-            _("Select Safe"),
-            self,
-            Gtk.FileChooserAction.OPEN,
-            None,
-            None,
-        )
+        if param and (path := param.get_string()):
+            self._open_database_in_window(path)
+            return
 
         keepass_filter = Gtk.FileFilter()
         keepass_filter.add_mime_type("application/x-keepass2")
         keepass_filter.add_pattern("*.kdbx")
-        # NOTE: KeePass + version number is a proper name, do not translate
-        keepass_filter.set_name(_("KeePass 3.1/4 Database"))
-        opening_dialog.add_filter(keepass_filter)
+        # TRANSLATORS Safe as in strongbox.
+        keepass_filter.set_name("KeePass 3.1/4 " + _("Safe"))
 
         binary_filter = Gtk.FileFilter()
         binary_filter.add_mime_type("application/octet-stream")
         binary_filter.set_name(_("Any file type"))
-        opening_dialog.add_filter(binary_filter)
 
-        opening_dialog.connect(
-            "response",
-            self._on_open_filechooser_response,
-            opening_dialog,
-        )
-        opening_dialog.show()
+        filters = Gio.ListStore.new(Gtk.FileFilter)
+        filters.append(keepass_filter)
+        filters.append(binary_filter)
 
-    def _on_open_filechooser_response(
-        self,
-        dialog: Gtk.Dialog,
-        response: Gtk.ResponseType,
-        _dialog: Gtk.Dialog,
-    ) -> None:
-        dialog.destroy()
-        if response == Gtk.ResponseType.ACCEPT:
-            db_gfile = dialog.get_file()
-            db_filename = db_gfile.get_path()
-            logging.debug("File selected: %s", db_filename)
+        dialog = Gtk.FileDialog.new()
+        dialog.props.title = _("Select Safe")
+        dialog.props.filters = filters
 
-            self._open_database_in_window(db_filename)
+        dialog.open(self, None, self._on_open_filechooser_response)
 
-        elif response == Gtk.ResponseType.CANCEL:
-            logging.debug("File selection canceled")
+    def _on_open_filechooser_response(self, dialog, result):
+        try:
+            db_gfile = dialog.open_finish(result)
+        except GLib.Error as err:
+            logging.debug("Could not open file: %s", err.message)
+        else:
+            if db_filename := db_gfile.get_path():
+                logging.debug("File selected: %s", db_filename)
+                self._open_database_in_window(db_filename)
+            else:
+                logging.error("File does not have a path, is in a virtual file system?")
+                self.send_notification(_("File does not have a valid path"))
 
     def _open_database_in_window(self, filepath):
         if self.unlocked_db:
             auto_save = gsecrets.config_manager.get_save_automatically()
             is_dirty = self.unlocked_db.database_manager.is_dirty
-            is_current = self.unlocked_db.database_manager.database_path == filepath
+            is_current = self.unlocked_db.database_manager.path == filepath
             is_locked = self.unlocked_db.database_manager.props.locked
 
             if is_locked:
                 if is_dirty and not is_current:
-                    app = Gio.Application.get_default()
+                    app = self.props.application
                     window = app.new_window()
                     window.start_database_opening_routine(filepath)
                     window.present()
@@ -206,20 +162,20 @@ class Window(Adw.ApplicationWindow):
                         return
 
                     if auto_save:
-                        self.unlocked_db.save_database()
+                        self.unlocked_db.auto_save_database()
 
-                    self.unlocked_db.do_dispose()
+                    self.unlocked_db.inner_dispose()
                     self.unlocked_db = None
                     self.start_database_opening_routine(filepath)
 
             else:
                 if not is_current:
-                    app = Gio.Application.get_default()
+                    app = self.props.application
                     window = app.new_window()
                     window.start_database_opening_routine(filepath)
                     window.present()
                 else:
-                    self.send_notification(_("Safe is Already Open"))
+                    self.send_notification(_("Safe is already open"))
 
         else:
             self.start_database_opening_routine(filepath)
@@ -231,6 +187,7 @@ class Window(Adw.ApplicationWindow):
         unlock_db = UnlockDatabase(self, database)
         self._unlock_database_bin.props.child = unlock_db
         self.view = self.View.UNLOCK_DATABASE
+        unlock_db.grab_entry_focus()
 
     #
     # Create Database Methods
@@ -238,34 +195,31 @@ class Window(Adw.ApplicationWindow):
 
     def on_new_database_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         """Callback function to create a new safe."""
-        creation_dialog = Gtk.FileChooserNative.new(
-            # NOTE: Filechooser title for creating a new keepass safe kdbx file
-            _("Create Safe"),
-            self,
-            Gtk.FileChooserAction.SAVE,
-            _("_Create"),
-            None,
-        )
-        creation_dialog.set_current_name(_("Safe") + ".kdbx")
-        creation_dialog.set_modal(True)
-
         filter_text = Gtk.FileFilter()
-        # NOTE: KeePass + version number is a proper name, do not translate
-        filter_text.set_name(_("KeePass 3.1/4 Database"))
+        # TRANSLATORS Safe as in strongbox.
+        filter_text.set_name("KeePass 3.1/4 " + _("Safe"))
         filter_text.add_mime_type("application/x-keepass2")
-        creation_dialog.add_filter(filter_text)
 
-        creation_dialog.connect(
-            "response", self._on_create_filechooser_response, creation_dialog
-        )
-        creation_dialog.show()
+        filters = Gio.ListStore.new(Gtk.FileFilter)
+        filters.append(filter_text)
 
-    def _on_create_filechooser_response(
-        self, dialog: Gtk.Dialog, response: Gtk.ResponseType, _dialog: Gtk.Dialog
-    ) -> None:
-        dialog.destroy()
-        if response == Gtk.ResponseType.ACCEPT:
-            filepath = dialog.get_file().get_path()
+        dialog = Gtk.FileDialog.new()
+        # NOTE: Filechooser title for creating a new keepass safe kdbx file
+        dialog.props.title = _("Create Safe")
+        dialog.props.filters = filters
+        dialog.props.accept_label = _("_Create")
+        # NOTE: Safe as in strongbox
+        dialog.props.initial_name = _("Safe") + ".kdbx"
+
+        dialog.save(self, None, self._on_create_filechooser_response)
+
+    def _on_create_filechooser_response(self, dialog, result):
+        try:
+            gfile = dialog.save_finish(result)
+        except GLib.Error as err:
+            logging.debug("Could not save file: %s", err.message)
+        else:
+            filepath = gfile.get_path()
             if self.unlocked_db:
                 auto_save = gsecrets.config_manager.get_save_automatically()
                 is_dirty = self.unlocked_db.database_manager.is_dirty
@@ -274,81 +228,71 @@ class Window(Adw.ApplicationWindow):
 
                 if is_open:
                     self.send_notification(
-                        _("Cannot Create Safe: Safe is Already Open")
+                        _("Cannot create Safe: Safe is already open")
                     )
                     return
 
                 if is_locked:
                     if is_dirty and not auto_save:
-                        app = Gio.Application.get_default()
+                        app = self.props.application
                         window = app.new_window()
-                        window.start_database_creation_routine(filepath)
+                        window.create_new_database(filepath)
                         window.present()
                     else:
                         if auto_save:
-                            self.unlocked_db.save_database()
+                            self.unlocked_db.auto_save_database()
 
-                        self.unlocked_db.do_dispose()
+                        self.unlocked_db.inner_dispose()
                         self.unlocked_db = None
                         self.create_new_database(filepath)
                 else:
-                    app = Gio.Application.get_default()
+                    app = self.props.application
                     window = app.new_window()
-                    window.start_database_creation_routine(filepath)
+                    window.create_new_database(filepath)
                     window.present()
 
             else:
                 self.create_new_database(filepath)
 
     def create_new_database(self, filepath: str) -> None:
-        """invoked in a separate thread to create a new safe."""
+        """Copy stock database onto filepath."""
         self._spinner.start()
-        self._main_view.set_visible_child(self._spinner)
-        creation_thread = threading.Thread(
-            target=self.start_database_creation_routine, args=[filepath]
-        )
-        creation_thread.daemon = True
-        creation_thread.start()
+        self._main_view.set_visible_child_name("spinner")
 
-    def start_database_creation_routine(self, filepath):
-        """Copy stock database onto filepath. Most functions are called
-        inside a idle_add, so they get executed in the main thread
-        rather than in the thread where the database is created."""
         stock_db_file: Gio.File = Gio.File.new_for_uri(
             "resource:///org/gnome/World/Secrets/database.kdbx"
         )
         new_db_file: Gio.File = Gio.File.new_for_path(filepath)
 
-        def database_created_callback(database_manager):
-            """Callback to be run on the main thread."""
-            self._spinner.stop()
-            create_database = CreateDatabase(self, database_manager)
-            self._create_database_bin.props.child = create_database
-            self.view = self.View.CREATE_DATABASE
-
-        def error_callback(err):
-            logging.debug("Could not copy new database: %s", err)
-            self._spinner.stop()
-            self.invoke_initial_screen()
-            self.send_notification(_("Could not Create new Safe"))
+        def unlock_callback(database_manager, result):
+            try:
+                database_manager.unlock_finish(result)
+            except GLib.Error as err:
+                logging.debug("Could not unlock safe: %s", err.message)
+                self.invoke_initial_screen()
+                self.send_notification(_("Could not create new Safe"))
+            else:
+                create_database = CreateDatabase(self, database_manager)
+                self._create_database_bin.props.child = create_database
+                self._create_view = create_database
+                self.view = self.View.CREATE_DATABASE
+            finally:
+                self._spinner.stop()
 
         def copy_callback(gfile, result):
             try:
-                success = gfile.copy_finish(result)
-                if not success:
-                    raise Exception("IO operation error")
-
-            except Exception as err:  # pylint: disable=broad-except
-                GLib.idle_add(error_callback, err)
+                gfile.copy_finish(result)
+            except GLib.Error as err:
+                logging.debug("Could not copy new database: %s", err.message)
+                self.invoke_initial_screen()
+                self.send_notification(_("Could not create new Safe"))
+                self._spinner.stop()
             else:
-                try:
-                    database_manager = DatabaseManager(
-                        filepath, "liufhre86ewoiwejmrcu8owe"
-                    )
-                except Exception as err:  # pylint: disable=broad-except
-                    GLib.idle_add(error_callback, err)
-                else:
-                    GLib.idle_add(database_created_callback, database_manager)
+                database_manager = DatabaseManager(filepath)
+                database_manager.unlock_async(
+                    "liufhre86ewoiwejmrcu8owe",
+                    callback=unlock_callback,
+                )
 
         stock_db_file.copy_async(
             new_db_file,
@@ -366,6 +310,15 @@ class Window(Adw.ApplicationWindow):
         logging.debug("Saving window geometry: (%s, %s)", width, height)
         gsecrets.config_manager.set_window_size([width, height])
 
+    def force_close(self):
+        """Method to close a window with unsaved changes.
+
+        Should only be called from SaveDialog and QuitConflictDialog."""
+        if unlocked_db := self.unlocked_db:
+            unlocked_db.database_manager.is_dirty = False
+
+        self.close()
+
     def do_close_request(self) -> bool:  # pylint: disable=arguments-differ
         """This is invoked when the window close button is pressed.
 
@@ -378,16 +331,41 @@ class Window(Adw.ApplicationWindow):
             self.save_window_size()
             return False
 
-        is_database_dirty = self.unlocked_db.database_manager.is_dirty
+        dbm = self.unlocked_db.database_manager
+        is_database_dirty = dbm.is_dirty
+
+        def on_save(dbm, result):
+            try:
+                is_saved = dbm.save_finish(result)
+            except GLib.Error as err:
+                logging.error("Could not save Safe: %s", err.message)
+                self.send_notification(_("Could not save Safe"))
+            else:
+                if is_saved:
+                    self.close()
+                else:
+                    # This shouldn't happen
+                    self.send_notification(_("Could not save Safe"))
+
+        def on_check_file_changes(dbm, result):
+            try:
+                conflicts = dbm.check_file_changes_finish(result)
+            except GLib.Error as err:
+                logging.error("Could not monitor file changes: %s", err.message)
+                self.send_notification(_("Could not save safe"))
+            else:
+                if conflicts:
+                    dialog = QuitConflictDialog(self, dbm, on_save)
+                    dialog.present()
+                else:
+                    if gsecrets.config_manager.get_save_automatically():
+                        self.unlocked_db.database_manager.save_async(on_save)
+                    else:
+                        save_dialog = SaveDialog(self)
+                        save_dialog.present()
 
         if is_database_dirty:
-            if gsecrets.config_manager.get_save_automatically():
-                self.unlocked_db.save_database()
-                self.save_window_size()
-                return False
-
-            save_dialog = SaveDialog(self)
-            save_dialog.show()
+            dbm.check_file_changes_async(on_check_file_changes)
             return True
 
         self.save_window_size()
@@ -419,7 +397,13 @@ class Window(Adw.ApplicationWindow):
         self.add_action(new_database_action)
         new_database_action.connect("activate", self.on_new_database_action)
 
-        open_database_action = Gio.SimpleAction.new("open_database", None)
+        go_back_action = Gio.SimpleAction.new("go_back", None)
+        self.add_action(go_back_action)
+        go_back_action.connect("activate", self.on_go_back_action)
+
+        open_database_action = Gio.SimpleAction.new(
+            "open_database", GLib.VariantType("s")
+        )
         self.add_action(open_database_action)
         open_database_action.connect("activate", self.on_open_database_action)
 
@@ -430,8 +414,8 @@ class Window(Adw.ApplicationWindow):
             "db.add_entry",
             "db.add_group",
             "db.settings",
-            "db.search",
-            "go_back",
+            "db.undo_delete",
+            "db.undo_attribute_delete",
             "element.delete",
             "entry.duplicate",
             "entry.references",
@@ -477,32 +461,28 @@ class Window(Adw.ApplicationWindow):
             action_db.show_database_settings()
         elif name == "db.selection":
             action_db.selection_mode_headerbar.on_selection_action(param)
+        elif name == "db.undo_delete":
+            action_db.undo_delete()
+        elif name == "db.undo_attribute_delete":
+            action_db.undo_attribute_delete()
         elif name in ["db.save", "db.save_dirty"]:
-            action_db.save_safe()
-        elif name == "go_back":
-            action_db.go_back()
+            action_db.save_database()
         elif name in ["db.add_entry", "db.add_group"]:
             if (
                 action_db.props.database_locked
                 or action_db.props.selection_mode
                 or action_db.in_edit_page
-                or action_db.props.search_active
             ):
                 return
 
+            if action_db.props.search_active:
+                action_db.props.search_active = False
+
             if name == "db.add_entry":
                 action_db.on_add_entry_action()
             else:
                 action_db.on_add_group_action()
 
-        elif name == "db.search":
-            if not (
-                action_db.props.database_locked
-                or action_db.props.selection_mode
-                or action_db.in_edit_page
-            ):
-                action_db.props.search_active = True
-
     def on_about_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         """Invoked when we click "about" in the main menu"""
         builder = Gtk.Builder.new_from_resource(
@@ -510,11 +490,20 @@ class Window(Adw.ApplicationWindow):
         )
         about_dialog = builder.get_object("about_dialog")
         about_dialog.set_transient_for(self)
-        about_dialog.show()
+        about_dialog.present()
 
     def on_settings_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         SettingsDialog(self).present()
 
+    def on_go_back_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
+        if self.view == self.View.CREATE_DATABASE:
+            if create_view := self._create_view:
+                create_view.go_back()
+        else:
+            if action_db := self.unlocked_db:
+                if not action_db.props.database_locked:
+                    action_db.go_back()
+
     @Gtk.Template.Callback()
     def on_back_button_pressed(
         self,
@@ -532,21 +521,14 @@ class Window(Adw.ApplicationWindow):
     @view.setter  # type: ignore
     def view(self, new_view: View) -> None:
         stack = self._main_view
-        headerbar_stack = self._headerbar_stack
 
         self._view = new_view
 
         if new_view == self.View.WELCOME:
             stack.props.visible_child_name = "welcome"
-            headerbar_stack.props.visible_child_name = "recent_files"
-        elif new_view == self.View.RECENT_FILES:
-            stack.props.visible_child_name = "recent_files"
-            headerbar_stack.props.visible_child_name = "recent_files"
         elif new_view == self.View.UNLOCK_DATABASE:
             stack.props.visible_child_name = "unlock_database"
-            headerbar_stack.props.visible_child_name = "unlock_database"
         elif new_view == self.View.UNLOCKED_DATABASE:
             stack.props.visible_child_name = "unlocked"
         elif new_view == self.View.CREATE_DATABASE:
             stack.props.visible_child_name = "create_database"
-            headerbar_stack.props.visible_child_name = "create_database"
