@@ -17,7 +17,7 @@ class Application(Adw.Application):
     application_id = const.APP_ID
     settings = Gio.Settings.new(application_id)
 
-    def __init__(self, *args, **_kwargs):
+    def __init__(self, executor, *args, **_kwargs):
         super().__init__(
             *args,
             application_id=self.application_id,
@@ -36,6 +36,8 @@ class Application(Adw.Application):
             None,
         )
 
+        self.executor = executor
+
     def do_startup(self):  # pylint: disable=arguments-differ
         Adw.Application.do_startup(self)
 
@@ -128,7 +130,7 @@ class Application(Adw.Application):
     def add_global_accelerators(self):
         self.set_accels_for_action("app.quit", ["<Control>q"])
         self.set_accels_for_action("win.settings", ["<Control>comma"])
-        self.set_accels_for_action("win.open_database", ["<Control>o"])
+        self.set_accels_for_action("win.open_database('')", ["<Control>o"])
         self.set_accels_for_action("win.new_database", ["<Control>n"])
         self.set_accels_for_action("win.db.save", ["<Control>s"])
         self.set_accels_for_action("win.db.search", ["<Control>f"])
@@ -136,7 +138,4 @@ class Application(Adw.Application):
         self.set_accels_for_action("win.db.add_entry", ["<Control>e"])
         self.set_accels_for_action("win.db.add_group", ["<Control>g"])
         self.set_accels_for_action("win.go_back", ["Escape"])
-        self.set_accels_for_action("win.entry.copy_password", ["<Control><Shift>c"])
-        self.set_accels_for_action("win.entry.copy_user", ["<Control><Shift>b"])
-        self.set_accels_for_action("win.entry.copy_otp", ["<Control><Shift>t"])
         self.set_accels_for_action("window.close", ["<Control>w"])
