@@ -6,7 +6,7 @@ import os
 import typing
 from gettext import gettext as _
 
-from gi.repository import Gio, GLib, Gtk
+from gi.repository import Adw, Gio, GLib, Gtk
 
 import gsecrets.config_manager
 from gsecrets import const
@@ -19,7 +19,7 @@ if typing.TYPE_CHECKING:
 
 
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/unlock_database.ui")
-class UnlockDatabase(Gtk.Box):
+class UnlockDatabase(Adw.Bin):
     # pylint: disable=too-many-instance-attributes
 
     __gtype_name__ = "UnlockDatabase"
@@ -35,8 +35,6 @@ class UnlockDatabase(Gtk.Box):
     keyfile_spinner = Gtk.Template.Child()
     keyfile_stack = Gtk.Template.Child()
     password_entry = Gtk.Template.Child()
-    spinner = Gtk.Template.Child()
-    spinner_stack = Gtk.Template.Child()
     status_page = Gtk.Template.Child()
     headerbar = Gtk.Template.Child()
     unlock_button = Gtk.Template.Child()
@@ -46,6 +44,8 @@ class UnlockDatabase(Gtk.Box):
 
         super().__init__()
 
+        self.spinner = Gtk.Spinner.new()
+
         filepath = database_file.get_path()
 
         self.window = window
@@ -67,29 +67,32 @@ class UnlockDatabase(Gtk.Box):
         if gsecrets.const.IS_DEVEL:
             self.status_page.props.icon_name = gsecrets.const.APP_ID
 
+    def do_unmap(self):  # pylint: disable=arguments-differ
+        Gtk.Widget.do_unmap(self)
+        self.keyfile_spinner.props.spinning = False
+        self.spinner.props.spinning = False
+
     def grab_entry_focus(self):
         self.password_entry.grab_focus()
 
     @Gtk.Template.Callback()
     def _on_keyfile_button_clicked(self, _widget):
         """cb invoked when we unlock a database via keyfile"""
-        dialog = Gtk.FileChooserNative.new(
-            _("Select Keyfile"), self.window, Gtk.FileChooserAction.OPEN, None, None
-        )
-        dialog.add_filter(KeyFileFilter().file_filter)
-        dialog.connect("response", self.on_dialog_response, dialog)
-        dialog.show()
+        filters = Gio.ListStore.new(Gtk.FileFilter)
+        filters.append(KeyFileFilter().file_filter)
 
-    def on_dialog_response(
-        self, dialog: Gtk.Dialog, response: Gtk.ResponseType, _dialog: Gtk.Dialog
-    ) -> None:
-        dialog.destroy()
-        if response == Gtk.ResponseType.ACCEPT:
-            keyfile = dialog.get_file()
-            if keyfile is None:
-                logging.debug("No file selected")
-                return
+        dialog = Gtk.FileDialog.new()
+        dialog.props.filters = filters
+        dialog.props.title = _("Select Keyfile")
 
+        dialog.open(self.window, None, self.on_dialog_response)
+
+    def on_dialog_response(self, dialog, result):
+        try:
+            keyfile = dialog.open_finish(result)
+        except GLib.Error as err:
+            logging.debug("Could not open file: %s", err.message)
+        else:
             self.set_keyfile(keyfile)
 
     def set_keyfile(self, keyfile: Gio.File) -> None:
@@ -189,7 +192,7 @@ class UnlockDatabase(Gtk.Box):
     #
 
     def _open_database(self):
-        self.spinner_stack.props.visible_child_name = "spinner"
+        self.unlock_button.props.child = self.spinner
         self.spinner.start()
 
         self._set_sensitive(False)
@@ -254,7 +257,7 @@ class UnlockDatabase(Gtk.Box):
 
     def _reset_unlock_button(self):
         self.spinner.stop()
-        self.spinner_stack.props.visible_child_name = "image"
+        self.unlock_button.props.icon_name = "changes-allow-symbolic"
 
     def _reset_page(self):
         self.keyfile_path = None
@@ -269,6 +272,10 @@ class UnlockDatabase(Gtk.Box):
         self._reset_unlock_button()
 
     def _set_sensitive(self, sensitive):
+        delegate = self.password_entry.get_delegate()
+        if delegate.has_focus() and not sensitive:
+            self.window.props.focus_widget = None
+
         self.password_entry.set_sensitive(sensitive)
         self.keyfile_button.set_sensitive(sensitive)
         self.unlock_button.set_sensitive(sensitive)
