@@ -3,12 +3,12 @@ from __future__ import annotations
 
 import logging
 import os
-import threading
 import typing
 from enum import IntEnum
 from gettext import gettext as _
+from gettext import ngettext
 
-from gi.repository import Gdk, Gio, GLib, GObject, Gtk
+from gi.repository import Adw, Gio, GLib, GObject, Gtk
 
 import gsecrets.config_manager
 from gsecrets import const
@@ -20,11 +20,10 @@ from gsecrets.pathbar import Pathbar
 from gsecrets.safe_element import SafeElement, SafeEntry, SafeGroup
 from gsecrets.unlocked_headerbar import UnlockedHeaderBar
 from gsecrets.widgets.database_settings_dialog import DatabaseSettingsDialog
-from gsecrets.widgets.edit_element_headerbar import EditElementHeaderbar, PageType
 from gsecrets.widgets.properties_dialog import PropertiesDialog
 from gsecrets.widgets.references_dialog import ReferencesDialog
+from gsecrets.widgets.saving_conflict_dialog import SavingConflictDialog
 from gsecrets.widgets.search import Search
-from gsecrets.widgets.search_headerbar import SearchHeaderbar
 from gsecrets.widgets.selection_mode_headerbar import SelectionModeHeaderbar
 from gsecrets.widgets.unlocked_database_page import UnlockedDatabasePage
 
@@ -33,28 +32,57 @@ if typing.TYPE_CHECKING:
     from gsecrets.widgets.window import Window
 
 
+class UndoData:
+    def __init__(self, elements, toast):
+        self.elements = elements
+        self.toast = toast
+
+
+class AttributeUndoData:
+    # pylint: disable=too-many-arguments
+    def __init__(
+        self,
+        toast: Adw.Toast,
+        entry: SafeEntry,
+        key: str,
+        value: str,
+        protected: bool = False,
+    ):
+        self.toast = toast
+        self.key = key
+        self.value = value
+        self.entry = entry
+        self.protected = protected
+
+
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/unlocked_database.ui")
-class UnlockedDatabase(Gtk.Box):
+class UnlockedDatabase(Adw.Bin):
     # pylint: disable=too-many-instance-attributes
     # pylint: disable=too-many-public-methods
     __gtype_name__ = "UnlockedDatabase"
 
     # Connection handlers
     db_locked_handler: int | None = None
-    db_save_handler: int | None = None
-    clipboard_timer_handler: int | None = None
+    # Used only to delete the clipboard if the window is not active.
+    on_is_active_handler: int | None = None
     _current_element: SafeElement | None = None
     _lock_timer_handler: int | None = None
     save_loop: int | None = None  # If int, a thread periodically saves the database
     session_handler_id: int | None = None
 
     action_bar = Gtk.Template.Child()
-    _edit_page_bin = Gtk.Template.Child()
+    _edit_page = Gtk.Template.Child()
+    headerbar_stack = Gtk.Template.Child()
+    _main_view = Gtk.Template.Child()
     _stack = Gtk.Template.Child()
-    _unlocked_db_deck = Gtk.Template.Child()
+    _navigation_view = Gtk.Template.Child()
     _unlocked_db_stack = Gtk.Template.Child()
+    search_bar = Gtk.Template.Child()
+    search_entry = Gtk.Template.Child()
 
     selection_mode = GObject.Property(type=bool, default=False)
+    undo_data: UndoData | None = None
+    attribute_undo_data: AttributeUndoData | None = None
 
     class Mode(IntEnum):
         ENTRY = 0
@@ -79,17 +107,11 @@ class UnlockedDatabase(Gtk.Box):
         self.action_bar.pack_start(mobile_pathbar)
 
         # Headerbars
-        self.edit_entry_headerbar = EditElementHeaderbar(self, PageType.ENTRY)
-        self.edit_group_headerbar = EditElementHeaderbar(self, PageType.GROUP)
-        self.search_headerbar = SearchHeaderbar()
         self.selection_mode_headerbar = SelectionModeHeaderbar(self)
         self.headerbar = UnlockedHeaderBar(self)
 
-        self.window.add_headerbar(self.edit_entry_headerbar)
-        self.window.add_headerbar(self.edit_group_headerbar)
-        self.window.add_headerbar(self.search_headerbar)
-        self.window.add_headerbar(self.selection_mode_headerbar)
-        self.window.add_headerbar(self.headerbar)
+        self.headerbar_stack.add_child(self.selection_mode_headerbar)
+        self.headerbar_stack.add_child(self.headerbar)
 
         # self.search has to be loaded after the search headerbar.
         self._search_active = False
@@ -98,25 +120,37 @@ class UnlockedDatabase(Gtk.Box):
         self.search.initialize()
 
         # Browser Mode
-        self.show_browser_page(self.current_element)
+        self.show_browser_page(self.current_element)  # type: ignore
 
         self.setup()
 
-        self.clipboard = Gdk.Display.get_default().get_clipboard()
+        self.clipboard = self.get_clipboard()
 
         self.connect("notify::selection-mode", self._on_selection_mode_changed)
         self.db_locked_handler = self.database_manager.connect(
             "notify::locked", self._on_database_lock_changed
         )
-        self.db_save_handler = self.database_manager.connect(
-            "save-notification", self.on_database_save_notification
-        )
 
         # Sets the menu's save button sensitive property.
         save_action = window.lookup_action("db.save_dirty")
         dbm.bind_property("is-dirty", save_action, "enabled")
 
-    def do_dispose(self):
+        search_action = Gio.PropertyAction.new("db.search", self, "search-active")
+        window.add_action(search_action)
+
+        def on_locked(dbm, _spec):
+            if dbm.locked:
+                self.search_bar.set_key_capture_widget(None)
+            else:
+                self.search_bar.set_key_capture_widget(window)
+
+        dbm.connect("notify::locked", on_locked)
+        self.search_bar.set_key_capture_widget(window)
+
+        self.search_bar.connect_entry(self.search_entry)
+        self.bind_property("search-active", self.search_bar, "search-mode-enabled")
+
+    def inner_dispose(self):
         logging.debug("Database disposed")
         self.cleanup()
 
@@ -124,18 +158,6 @@ class UnlockedDatabase(Gtk.Box):
             self.database_manager.disconnect(self.db_locked_handler)
             self.db_locked_handler = None
 
-        if self.db_save_handler:
-            self.database_manager.disconnect(self.db_save_handler)
-            self.db_save_handler = None
-
-        self.search.do_dispose()
-
-        self.edit_entry_headerbar.unparent()
-        self.edit_group_headerbar.unparent()
-        self.search_headerbar.unparent()
-        self.selection_mode_headerbar.unparent()
-        self.headerbar.unparent()
-
     def setup(self):
         self.start_save_loop()
         self.start_database_lock_timer()
@@ -147,9 +169,13 @@ class UnlockedDatabase(Gtk.Box):
 
     def listbox_row_factory(self, element: SafeElement) -> Gtk.Widget:
         if element.is_entry:
-            return EntryRow(self, element)
+            row = EntryRow(self)
+            row.props.safe_entry = element
+            return row
 
-        return GroupRow(self, element)
+        row = GroupRow(self)
+        row.props.safe_group = element
+        return row
 
     def show_edit_page(self, element: SafeElement, new: bool = False) -> None:
         self.start_database_lock_timer()
@@ -162,33 +188,33 @@ class UnlockedDatabase(Gtk.Box):
         element.touch()
 
         if element.is_group:
-            page = GroupPage(self)
+            page = GroupPage(self, element)
             self.props.mode = self.Mode.GROUP_EDIT
         else:
-            page = EntryPage(self, new)
+            page = EntryPage(self, element, new)
             self.props.mode = self.Mode.ENTRY
 
-        self._edit_page_bin.set_child(page)
-        self._unlocked_db_deck.set_visible_child(self._edit_page_bin)
-
-        # Grab the focus of the Title entry.
-        # It is done here so that the Entry has a Window.
-        page.name_property_value_entry.grab_focus_without_selecting()
+        self._edit_page.set_child(page)
+        self._navigation_view.push(self._edit_page)
 
     def show_browser_page(self, group: SafeGroup) -> None:
         self.start_database_lock_timer()
-        page_name = group.uuid.urn
+        self._unlocked_db_stack.set_visible_child(self._stack)
+        if self._navigation_view.props.visible_page == self._edit_page:
+            # pop triggers _on_edit_page_hidden via the hidden signal.
+            self._navigation_view.pop()
+        else:
+            self._set_current_element_after_pop(group)
 
-        page = self._stack.get_child_by_name(page_name)
-        if page:
+    def _set_current_element_after_pop(self, group):
+        page_name = group.uuid.urn
+        if page := self._stack.get_child_by_name(page_name):
             self.props.current_element = page.group
         else:
             self.props.current_element = group
             new_page = UnlockedDatabasePage(self, group)
             self._stack.add_named(new_page, page_name)
 
-        self._unlocked_db_stack.set_visible_child(self._stack)
-        self._unlocked_db_deck.set_visible_child(self._unlocked_db_stack)
         if not self.props.selection_mode:
             self.props.mode = self.Mode.GROUP
 
@@ -200,32 +226,18 @@ class UnlockedDatabase(Gtk.Box):
         the Group edit page or Entry edit page."""
 
         boolean: bool = (
-            self._unlocked_db_deck.props.visible_child == self._edit_page_bin
+            self._navigation_view.props.visible_page == self._edit_page
         )
         return boolean
 
-    #
-    # Headerbar
-    #
-
-    def _update_headerbar(self) -> None:
-        """Display the correct headerbar according to search state."""
-        if self.props.mode == self.Mode.SEARCH:
-            self.window.set_headerbar(self.search.headerbar)
-        elif self.props.mode == self.Mode.GROUP_EDIT:
-            self.window.set_headerbar(self.edit_group_headerbar)
-        elif self.props.mode == self.Mode.ENTRY:
-            self.window.set_headerbar(self.edit_entry_headerbar)
-        elif self.props.mode == self.Mode.SELECTION:
-            self.window.set_headerbar(self.selection_mode_headerbar)
-        else:
-            self.window.set_headerbar(self.headerbar)
-
     def _on_selection_mode_changed(
         self, _unlocked_database: UnlockedDatabase, _value: GObject.ParamSpec
     ) -> None:
         if self.props.selection_mode:
             self.props.mode = self.Mode.SELECTION
+            self.headerbar_stack.set_visible_child(self.selection_mode_headerbar)
+        else:
+            self.headerbar_stack.set_visible_child(self.headerbar)
 
     #
     # Group and Entry Management
@@ -248,6 +260,10 @@ class UnlockedDatabase(Gtk.Box):
         element_uuid = self.props.current_element.uuid
         return self._stack.get_child_by_name(element_uuid.urn)
 
+    def delete_page(self, element):
+        if page := self._stack.get_child_by_name(element.uuid.urn):
+            self._stack.remove(page)
+
     #
     # Events
     #
@@ -270,93 +286,186 @@ class UnlockedDatabase(Gtk.Box):
             safe_entry = list_box_row.safe_entry
             self.show_edit_page(safe_entry)
 
-    def on_database_save_notification(
-        self, _database_manager: DatabaseManager, saved: bool
+    def on_save(
+        self, database_manager: DatabaseManager, result: Gio.AsyncResult
     ) -> None:
-        if saved:
-            self.window.send_notification(_("Safe Saved"))
+        try:
+            is_saved = database_manager.save_finish(result)
+        except GLib.Error as err:
+            logging.error("Could not save Safe: %s", err.message)
+            self.window.send_notification(_("Could not save Safe"))
         else:
-            self.window.send_notification(_("Could not Save Safe"))
-
-    def save_safe(self):
-        if self.database_manager.is_dirty is True:
-            if self.database_manager.save_running is False:
-                self.save_database(notification=True)
+            if is_saved:
+                self.window.send_notification(_("Safe saved"))
             else:
-                # NOTE: In-app notification to inform the user that
-                # already an unfinished save job is running
-                self.window.send_notification(
-                    _("Please Wait. Another Save is Running.")
-                )
-        else:
-            # NOTE: In-app notification to inform the user that no save
-            # is necessary because there where no changes made
-            self.window.send_notification(_("No Changes Made"))
+                self.window.send_notification(_("No changes made"))
+
+    def on_auto_save(
+        self, database_manager: DatabaseManager, result: Gio.AsyncResult
+    ) -> None:
+        # TODO Does it make sense to present this info to the user?
+        try:
+            database_manager.save_finish(result)
+        except GLib.Error as err:
+            logging.error("Could not automatically save Safe %s", err.message)
 
     def lock_safe(self):
         self.database_manager.props.locked = True
 
     def on_add_entry_action(self) -> None:
         """CB when the Add Entry menu was clicked"""
-        group = self.props.current_element.group
-        new_entry: SafeEntry = self.database_manager.add_entry_to_database(group)
+        group = self.props.current_element
+        new_entry: SafeEntry = group.new_entry()
         self.show_edit_page(new_entry, new=True)
 
     def on_add_group_action(self) -> None:
         """CB when menu entry Add Group is clicked"""
-        self.database_manager.is_dirty = True
-        safe_group = self.database_manager.add_group_to_database(
-            "", "0", "", self.props.current_element.group
-        )
+        group = self.props.current_element
+        safe_group = group.new_subgroup()
         self.show_edit_page(safe_group)
 
     def on_element_delete_action(self) -> None:
         """Delete the visible entry from the menu."""
+        element = self.props.current_element
         parent_group = self.props.current_element.parentgroup
-        uuid = self.props.current_element.element.uuid.urn
-        self.database_manager.delete_from_database(self.props.current_element.element)
+        if element.trash():
+            self.delete_page(element)
+            elements = []
+        else:
+            elements = [(element, parent_group)]
+
+        self.deleted_notification(elements)
 
         self.show_browser_page(parent_group)
 
-        page = self._stack.get_child_by_name(uuid)
-        if page:
-            self._stack.remove(page)
+    def deleted_notification(self, elements):
+        n_del = len(elements)
+        if n_del:
+            label = ngettext(
+                # pylint: disable=consider-using-f-string
+                "Element deleted",
+                "{} Elements deleted".format(n_del),
+                n_del,
+            )
+        else:
+            label = _("Deletion completed")
+
+        toast = Adw.Toast.new(label)
+
+        if n_del:
+            toast.props.button_label = _("Undo")
+            toast.props.action_name = "win.db.undo_delete"
+
+        if data := self.undo_data:
+            data.toast.dismiss()
+
+        def dismissed_db(toast):
+            if data := self.undo_data:
+                if data.toast == toast:
+                    self.undo_data = None
+
+        toast.connect("dismissed", dismissed_db)
+
+        self.undo_data = UndoData(elements, toast)
+        self.window.toast_overlay.add_toast(toast)
+
+    def undo_delete(self):
+        if data := self.undo_data:
+            for element, element_parent in data.elements:
+                element.move_to(element_parent)
+
+            self.undo_data = None
+
+    def attribute_deleted(self, entry, key, value, protected):
+        toast = Adw.Toast.new(_("Attribute deleted"))
+        toast.props.button_label = _("Undo")
+        toast.props.action_name = "win.db.undo_attribute_delete"
+
+        if data := self.attribute_undo_data:
+            data.toast.dismiss()
+
+        undo_data = AttributeUndoData(toast, entry, key, value, protected)
+        self.attribute_undo_data = undo_data
+        self.window.toast_overlay.add_toast(toast)
+
+    def undo_attribute_delete(self):
+        if data := self.attribute_undo_data:
+            data.toast.dismiss()
+            data.entry.set_attribute(data.key, data.value, data.protected)
+
+            self.attribute_undo_data = None
 
     def on_entry_duplicate_action(self):
-        self.database_manager.duplicate_entry(self.props.current_element.entry)
+        self.props.current_element.duplicate()
         parent_group = self.props.current_element.parentgroup
 
         self.show_browser_page(parent_group)
 
-    def send_to_clipboard(self, text: str, message: str = "") -> None:
+    def send_to_clipboard(
+        self,
+        text: str,
+        message: str = "",
+        toast_overlay: Adw.ToastOverlay | None = None,
+    ) -> None:
         if not message:
             message = _("Copied")
 
         self.start_database_lock_timer()
 
-        if self.clipboard_timer_handler:
-            GLib.source_remove(self.clipboard_timer_handler)
-            self.clipboard_timer_handler = None
+        if self.on_is_active_handler:
+            self.disconnect(self.on_is_active_handler)
+            self.on_is_active_handler = None
 
         self.clipboard.set(text)
 
-        self.window.send_notification(message)
+        if toast_overlay:
+            window = toast_overlay.get_root()
+            toast = Adw.Toast.new(message)
+            toast_overlay.add_toast(toast)
+        else:
+            window = self.window
+            self.window.send_notification(message)
 
         clear_clipboard_time = gsecrets.config_manager.get_clear_clipboard()
+        clipboard_last_content = self.clipboard.get_content()
 
         def callback():
-            self.clipboard_timer_handler = None
-            self.clipboard.set_content(None)
+            if window.props.is_active:
+                if clipboard_last_content == self.clipboard.get_content():
+                    logging.debug("Clipboard cleared")
+                    self.clipboard.set_content(None)
+            else:
+                logging.debug(
+                    "Could not clear the clipboard, it will clear on new focus"
+                )
 
-        self.clipboard_timer_handler = GLib.timeout_add_seconds(
-            clear_clipboard_time, callback
-        )
+                def on_is_active(window, _pspec):
+                    if self.on_is_active_handler:
+                        window.disconnect(self.on_is_active_handler)
+                        self.on_is_active_handler = None
+
+                    # Only clear clipboard if we set ourselves the content.
+                    if clipboard_last_content == self.clipboard.get_content():
+                        logging.debug("Clipboard cleared")
+                        self.clipboard.set_content(None)
+
+                self.on_is_active_handler = window.connect(
+                    "notify::is-active", on_is_active
+                )
+
+            return GLib.SOURCE_REMOVE
+
+        GLib.timeout_add_seconds(clear_clipboard_time, callback)
 
     def show_database_settings(self) -> None:
         DatabaseSettingsDialog(self).present()
 
     def on_session_lock(self, app: Gtk.Application, _pspec: GObject.ParamSpec) -> None:
-        if app.props.screensaver_active and not self.database_manager.props.locked:
+        if (
+            app.props.screensaver_active
+            and not self.database_manager.props.locked
+            and gsecrets.config_manager.get_lock_on_session_lock()
+        ):
             self.lock_timeout_database()
 
     #
@@ -384,20 +493,19 @@ class UnlockedDatabase(Gtk.Box):
         if locked:
             self.cleanup()
             if gsecrets.config_manager.get_save_automatically():
-                self.save_database()
+                self.auto_save_database()
 
-            filepath = self.database_manager.database_path
+            filepath = self.database_manager.path
             self.window.start_database_opening_routine(filepath)
         else:
             self.window.view = self.window.View.UNLOCKED_DATABASE
             self.setup()
-            self._update_headerbar()
 
     def lock_timeout_database(self):
         self.database_manager.props.locked = True
 
         # NOTE: Notification that a safe has been locked.
-        self.window.send_notification(_("Safe Locked due to Inactivity"))
+        self.window.send_notification(_("Safe locked due to inactivity"))
 
     #
     # Helper Methods
@@ -412,12 +520,7 @@ class UnlockedDatabase(Gtk.Box):
 
         This is the opposite of setup().
         """
-        logging.debug("Cleaning database %s", self.database_manager.database_path)
-
-        if self.clipboard_timer_handler:
-            GLib.source_remove(self.clipboard_timer_handler)
-            self.clipboard_timer_handler = None
-            self.clipboard.set_content(None)
+        logging.debug("Cleaning database %s", self.database_manager.path)
 
         if self._lock_timer_handler:
             GLib.source_remove(self._lock_timer_handler)
@@ -439,12 +542,9 @@ class UnlockedDatabase(Gtk.Box):
         # Cleanup temporal files created when opening attachments.
         def callback(gfile, result):
             try:
-                success = gfile.delete_finish(result)
-                if not success:
-                    raise Exception("IO operation error")
-
-            except Exception as err:  # pylint: disable=broad-except
-                logging.debug("Could not delete temporal file: %s", err)
+                gfile.delete_finish(result)
+            except GLib.Error as err:
+                logging.debug("Could not delete temporal file: %s", err.message)
 
         cache_dir = os.path.join(GLib.get_user_cache_dir(), const.SHORT_NAME, "tmp")
         for root, _dirs, files in os.walk(cache_dir):
@@ -453,22 +553,38 @@ class UnlockedDatabase(Gtk.Box):
                 gfile = Gio.File.new_for_path(file_path)
                 gfile.delete_async(GLib.PRIORITY_DEFAULT, None, callback)
 
-    def save_database(self, notification: bool = False) -> None:
+    def save_database(self) -> None:
         """Save the database.
 
-        If auto_save is False, a dialog asking for confirmation
-        will be displayed.
+        Shows a notification after saving.
         """
-        if not self.database_manager.is_dirty or self.database_manager.save_running:
-            return
 
-        save_thread = threading.Thread(
-            target=self.database_manager.save_database, args=[notification]
-        )
-        save_thread.daemon = False
-        save_thread.start()
+        def on_check_file_changes(dbm, result):
+            self.on_check_file_changes(dbm, result, self.on_save)
 
-        logging.debug("Saving database %s", self.database_manager.database_path)
+        self.database_manager.check_file_changes_async(on_check_file_changes)
+
+    def auto_save_database(self) -> None:
+        """Save the database."""
+        logging.debug("Automatically saving database")
+
+        def on_check_file_changes(dbm, result):
+            self.on_check_file_changes(dbm, result, self.on_auto_save)
+
+        self.database_manager.check_file_changes_async(on_check_file_changes)
+
+    def on_check_file_changes(self, dbm, result, on_save_callback):
+        try:
+            conflicts = dbm.check_file_changes_finish(result)
+        except GLib.Error as err:
+            logging.error("Could not monitor file changes: %s", err.message)
+            self.window.send_notification(_("Could not save Safe"))
+        else:
+            if conflicts:
+                dialog = SavingConflictDialog(self.window, dbm, on_save_callback)
+                dialog.present()
+            else:
+                self.database_manager.save_async(on_save_callback)
 
     def start_database_lock_timer(self):
         if self._lock_timer_handler:
@@ -491,8 +607,7 @@ class UnlockedDatabase(Gtk.Box):
     def threaded_save_loop(self) -> bool:
         """Saves the safe as long as it returns True."""
         if gsecrets.config_manager.get_save_automatically():
-            logging.debug("Automatically saving")
-            self.save_database()
+            self.auto_save_database()
 
         return GLib.SOURCE_CONTINUE
 
@@ -515,8 +630,7 @@ class UnlockedDatabase(Gtk.Box):
     def search_active(self) -> bool:
         """Property to know if search is active.
 
-        It is used by Search to update the widgets (mainly the
-        headerbar) accordingly.
+        It is used by Search to update the widgets accordingly.
 
         :returns: True is search is active
         :rtype: bool
@@ -529,6 +643,13 @@ class UnlockedDatabase(Gtk.Box):
 
         :param value: new search_active
         """
+        if (
+            self.database_manager.props.locked
+            or self.selection_mode
+            or self.in_edit_page
+        ):
+            return
+
         self._search_active = value
         if self._search_active:
             self.props.mode = self.Mode.SEARCH
@@ -536,6 +657,8 @@ class UnlockedDatabase(Gtk.Box):
         else:
             self.show_browser_page(self.current_element)
 
+    # TODO This property does not emit a notify signal when the manager locked
+    # property is updated.
     @GObject.Property(type=bool, default=False)
     def database_locked(self):
         """Get database lock status
@@ -552,4 +675,9 @@ class UnlockedDatabase(Gtk.Box):
     @mode.setter  # type: ignore
     def mode(self, new_mode: int) -> None:
         self._mode = new_mode
-        self._update_headerbar()
+
+    @Gtk.Template.Callback()
+    def _on_edit_page_hidden(self, _page):
+        # Makes sure that current_element is set correctly.
+        parent = self.props.current_element.parentgroup
+        self._set_current_element_after_pop(parent)
