@@ -1,20 +1,19 @@
 # SPDX-License-Identifier: GPL-3.0-only
 from __future__ import annotations
 
-import io
 import logging
 from pathlib import Path
-from uuid import UUID
 
 from gi.repository import Gio, GLib, GObject
 from pykeepass import PyKeePass
 
-from gsecrets.safe_element import SafeElement
+import gsecrets.config_manager as config
+from gsecrets.safe_element import SafeEntry, SafeGroup
 
 QUARK = GLib.quark_from_string("secrets")
 
 
-class DatabaseManager(GObject.GObject):
+class DatabaseManager(GObject.Object):
     # pylint: disable=too-many-public-methods
     # pylint: disable=too-many-instance-attributes
 
@@ -31,20 +30,31 @@ class DatabaseManager(GObject.GObject):
     """
 
     # self.db contains a `PyKeePass` database
+    _elements_loaded = False
     password_try = ""
     _is_dirty = False  # Does the database need saving?
     save_running = False
     _opened = False
 
+    # Used for checking if there are changes in the file.
+    file_size: int | None = None
+    file_mtime: int | None = None
+
     # Only used for setting the credentials to their actual values in case of
     # errors.
     old_password: str = ""
     old_keyfile: str = ""
     old_keyfile_hash: str = ""
 
+    trash_bin: SafeGroup | None = None
+    root: SafeGroup | None = None
+
     locked = GObject.Property(type=bool, default=False)
     is_dirty = GObject.Property(type=bool, default=False)
 
+    # To be emitted when the elements list model needs to be re sorted.
+    sorting_changed = GObject.Signal(arg_types=(bool,))
+
     def __init__(self, database_path: str) -> None:
         """Initialize the database handling logic.
 
@@ -52,6 +62,9 @@ class DatabaseManager(GObject.GObject):
         """
         super().__init__()
 
+        self.entries = Gio.ListStore.new(SafeEntry)
+        self.groups = Gio.ListStore.new(SafeGroup)
+
         self._path = database_path
         self.db: PyKeePass = None
         self.keyfile_hash: str = ""
@@ -63,7 +76,7 @@ class DatabaseManager(GObject.GObject):
         keyfile_hash: str = "",
         callback: Gio.AsyncReadyCallback = None,
     ) -> None:
-        """Unlocks an opens a safe.
+        """Unlocks and opens a safe.
 
         This pykeepass to open a safe database. If the database cannot
         be opened, an exception is raised.
@@ -91,6 +104,7 @@ class DatabaseManager(GObject.GObject):
                 task.return_error(err)
             else:
                 self.keyfile_hash = keyfile_hash
+                self._update_file_monitor()
                 task.return_value(db)
 
         task = Gio.Task.new(self, None, callback)
@@ -101,10 +115,16 @@ class DatabaseManager(GObject.GObject):
             _success, db = result.propagate_value()
         except GLib.Error as err:
             raise err
-        else:
-            self.db = db
-            self._opened = True
-            logging.debug("Opening of safe %s was successful", self.path)
+
+        self.db = db
+        self._opened = True
+        logging.debug("Opening of safe %s was successful", self.path)
+
+        if not self._elements_loaded:
+            self.entries.splice(0, 0, [SafeEntry(self, e) for e in db.entries])
+            self.groups.splice(0, 0, [SafeGroup(self, g) for g in db.groups])
+
+            self._elements_loaded = True
 
     #
     # Database Modifications
@@ -124,8 +144,6 @@ class DatabaseManager(GObject.GObject):
         task.run_in_thread(self._save_task)
 
     def _save_task(self, task, _obj, _data, _cancellable):
-        stream = io.BytesIO()
-
         if self.save_running:
             task.return_boolean(False)
             logging.debug("Save already running")
@@ -140,30 +158,13 @@ class DatabaseManager(GObject.GObject):
         self.save_running = True
 
         try:
-            self.db.save(filename=stream)
+            self.db.save()
         except Exception as err:  # pylint: disable=broad-except
             err = GLib.Error.new_literal(QUARK, str(err), 2)
             task.return_error(err)
         else:
-            owned_bytes = stream.getvalue()
-            if owned_bytes is None:
-                error = GLib.Error.new_literal(QUARK, "Stream is empty", 3)
-                task.return_error(error)
-                return
-
-            gfile = Gio.File.new_for_path(self.path)
-            try:
-                gfile.replace_contents(
-                    owned_bytes,
-                    None,
-                    False,
-                    Gio.FileCreateFlags.NONE,
-                    None,
-                )
-            except GLib.Error as err:
-                task.return_error(err)
-            else:
-                task.return_boolean(True)
+            self._update_file_monitor()
+            task.return_boolean(True)
 
     def save_finish(self, result: Gio.AsyncResult) -> bool:
         """Finishes save_async, returns whether the safe was saved.
@@ -173,12 +174,12 @@ class DatabaseManager(GObject.GObject):
             is_saved = result.propagate_boolean()
         except GLib.Error as err:
             raise err
-        else:
-            self.is_dirty = False
-            if is_saved:
-                logging.debug("Database %s saved successfully", self.path)
 
-            return is_saved
+        self.is_dirty = False
+        if is_saved:
+            logging.debug("Database %s saved successfully", self.path)
+
+        return is_saved
 
     def set_credentials_async(
         self, password, keyfile="", keyfile_hash="", callback=None
@@ -215,12 +216,90 @@ class DatabaseManager(GObject.GObject):
             self.keyfile_hash = self.old_keyfile_hash
 
             raise err
-        else:
-            self.is_dirty = False
-            if is_saved:
-                logging.debug("Credentials changed successfully")
 
-            return is_saved
+        self.is_dirty = False
+        if is_saved:
+            logging.debug("Credentials changed successfully")
+
+        return is_saved
+
+    def add_to_history(self) -> None:
+        # Add database uri to history.
+        uri = Gio.File.new_for_path(self._path).get_uri()
+        uri_list = config.get_last_opened_list()
+
+        if uri in uri_list:
+            uri_list.sort(key=uri.__eq__)
+        else:
+            uri_list.append(uri)
+
+        config.set_last_opened_list(uri_list)
+
+        # Set last opened database.
+        config.set_last_opened_database(uri)
+
+        # Add keyfile path to history.
+        if not config.get_remember_composite_key():
+            return
+
+        def filter_func(pair):
+            return pair[0] != uri
+
+        keyfile_pairs = config.get_last_used_composite_key()
+        new_pairs = list(filter(filter_func, keyfile_pairs))
+
+        if self.keyfile:
+            new_pairs.append([uri, self.keyfile])
+
+        config.set_last_used_composite_key(new_pairs)
+
+    def _update_file_monitor(self):
+        """Updates the modified time and size of the database file, this is a
+        blocking operation."""
+        gfile = Gio.File.new_for_path(self._path)
+        attributes = (
+            f"{Gio.FILE_ATTRIBUTE_STANDARD_SIZE},{Gio.FILE_ATTRIBUTE_TIME_MODIFIED}"
+        )
+        try:
+            info = gfile.query_info(attributes, Gio.FileQueryInfoFlags.NONE, None)
+        except GLib.Error as err:
+            logging.error("Could not read file size: %s", err.message)
+        else:
+            self.file_size = info.get_size()
+            self.file_mtime = info.get_modification_date_time().to_unix()
+
+    def check_file_changes_async(self, callback: Gio.AsyncReadyCallback) -> None:
+        task = Gio.Task.new(self, None, callback)
+        task.run_in_thread(self._check_file_changes_task)
+
+    def _check_file_changes_task(self, task, _obj, _data, _cancellable):
+        gfile = Gio.File.new_for_path(self._path)
+        attributes = (
+            f"{Gio.FILE_ATTRIBUTE_STANDARD_SIZE},{Gio.FILE_ATTRIBUTE_TIME_MODIFIED}"
+        )
+        try:
+            info = gfile.query_info(attributes, Gio.FileQueryInfoFlags.NONE, None)
+        except GLib.Error as err:
+            err = GLib.Error.new_literal(QUARK, str(err), 3)
+            task.return_error(err)
+        else:
+            if (
+                self.file_size != info.get_size()
+                or self.file_mtime != info.get_modification_date_time().to_unix()
+            ):
+                task.return_boolean(True)
+            else:
+                task.return_boolean(False)
+
+    def check_file_changes_finish(self, result: Gio.AsyncResult) -> bool:
+        """Finishes check_file_changes_async.
+        Returns whether the file was changed, can raise GLib.Error."""
+        try:
+            changed = result.propagate_boolean()
+        except GLib.Error as err:
+            raise err
+
+        return changed
 
     @property
     def password(self) -> str:
@@ -288,31 +367,6 @@ class DatabaseManager(GObject.GObject):
         """returns the database version"""
         return self.db.version
 
-    @GObject.Signal(
-        arg_types=(
-            SafeElement,
-            object,
-        )
-    )
-    def element_added(self, _element: SafeElement, _parent_uuid: UUID) -> None:
-        """Signal emitted when a new element was added to the database
-        it carries the UUID in string format of the parent group to which
-        the entry was added."""
-        logging.debug("Added new element to safe")
-
-    @GObject.Signal(arg_types=(object,))
-    def element_removed(self, element_uuid: UUID) -> None:
-        logging.debug("Element %s removed from safe", element_uuid)
-
-    @GObject.Signal(arg_types=(SafeElement, object, object))
-    def element_moved(
-        self,
-        _moved_element: SafeElement,
-        _old_location_uuid: UUID,
-        _new_location_uuid: UUID,
-    ) -> None:
-        logging.debug("Element moved safe")
-
     @property
     def opened(self) -> bool:
         return self._opened
