@@ -7,9 +7,9 @@ from gettext import gettext as _
 from gi.repository import Adw, GLib, GObject, Gtk
 
 from gsecrets.safe_element import EntryColor
+from gsecrets.safe_element import SafeEntry
 
 if typing.TYPE_CHECKING:
-    from gsecrets.safe_element import SafeEntry
     from gsecrets.unlocked_database import UnlockedDatabase
 
 
@@ -22,29 +22,33 @@ class EntryRow(Adw.ActionRow):
     _entry_icon = Gtk.Template.Child()
     selection_checkbox = Gtk.Template.Child()
 
-    def __init__(self, database: UnlockedDatabase, safe_entry: SafeEntry) -> None:
+    _safe_entry = None
+
+    def __init__(self, database: UnlockedDatabase) -> None:
         super().__init__()
 
-        self._safe_entry: SafeEntry = safe_entry
+        self._signals = GObject.SignalGroup.new(SafeEntry)
+        self._bindings = GObject.BindingGroup.new()
 
-        self.unlocked_database = database
-        self.db_manager = database.database_manager
-
-        self.assemble_entry_row()
-
-    def assemble_entry_row(self):
-        self._safe_entry.bind_property(
+        self._bindings.bind(
             "icon-name", self._entry_icon, "icon-name", GObject.BindingFlags.SYNC_CREATE
         )
 
-        self._safe_entry.connect("notify::name", self._on_entry_name_changed)
-        self._on_entry_name_changed(self._safe_entry, None)
+        self._signals.connect_closure(
+            "notify::name", self._on_entry_name_changed, False
+        )
+        self._signals.connect_closure(
+            "notify::username", self._on_entry_username_changed, False
+        )
+        self._signals.connect_closure(
+            "notify::color", self._on_entry_color_changed, False
+        )
+        self._signals.connect_closure(
+            "notify::expired", self._on_entry_notify_expired, False
+        )
 
-        self._safe_entry.connect("notify::username", self._on_entry_username_changed)
-        self._on_entry_username_changed(self._safe_entry, None)
-
-        self._safe_entry.connect("notify::color", self._on_entry_color_changed)
-        self._on_entry_color_changed(self._safe_entry, None)
+        self.unlocked_database = database
+        self.db_manager = database.database_manager
 
         # Selection Mode Checkboxes
         self.unlocked_database.bind_property(
@@ -54,12 +58,21 @@ class EntryRow(Adw.ActionRow):
             GObject.BindingFlags.SYNC_CREATE,
         )
 
-        # Expiration Date
-        self.safe_entry.connect(
-            "notify::expired",
-            self._on_entry_notify_expired,
-        )
-        self._on_entry_notify_expired(self.safe_entry, None)
+    @GObject.Property(type=SafeEntry)
+    def safe_entry(self):
+        return self._safe_entry
+
+    @safe_entry.setter  # type: ignore
+    def safe_entry(self, element):
+        self._safe_entry = element
+
+        self._signals.props.target = element
+        self._bindings.props.source = element
+
+        self._on_entry_name_changed(element, None)
+        self._on_entry_username_changed(element, None)
+        self._on_entry_color_changed(element, None)
+        self._on_entry_notify_expired(element, None)
 
     def _on_entry_notify_expired(self, safe_entry, _gparam):
         if safe_entry.expired and safe_entry.props.expires:
@@ -87,10 +100,6 @@ class EntryRow(Adw.ActionRow):
                 db_view.props.selection_mode = True
                 self.selection_checkbox.props.active = True
 
-    @property
-    def safe_entry(self) -> SafeEntry:
-        return self._safe_entry
-
     @Gtk.Template.Callback()
     def on_selection_checkbox_toggled(self, _widget):
         self.unlocked_database.start_database_lock_timer()
@@ -115,9 +124,9 @@ class EntryRow(Adw.ActionRow):
         )
 
     def _on_entry_name_changed(
-        self, _safe_entry: SafeEntry, _value: GObject.ParamSpec
+        self, safe_entry: SafeEntry, _value: GObject.ParamSpec
     ) -> None:
-        entry_name = GLib.markup_escape_text(self._safe_entry.props.name)
+        entry_name = GLib.markup_escape_text(safe_entry.props.name)
         if entry_name:
             self.remove_css_class("italic-title")
             self.props.title = entry_name
@@ -126,9 +135,9 @@ class EntryRow(Adw.ActionRow):
             self.props.title = _("Title not Specified")
 
     def _on_entry_username_changed(
-        self, _safe_entry: SafeEntry, _value: GObject.ParamSpec
+        self, safe_entry: SafeEntry, _value: GObject.ParamSpec
     ) -> None:
-        entry_username = GLib.markup_escape_text(self._safe_entry.props.username)
+        entry_username = GLib.markup_escape_text(safe_entry.props.username)
         if entry_username:
             self.remove_css_class("italic-subtitle")
             self.props.subtitle = entry_username
@@ -137,13 +146,13 @@ class EntryRow(Adw.ActionRow):
             self.props.subtitle = _("Username not specified")
 
     def _on_entry_color_changed(
-        self, _safe_entry: SafeEntry, _value: GObject.ParamSpec
+        self, safe_entry: SafeEntry, _value: GObject.ParamSpec
     ) -> None:
         # Clear current style
         for color in EntryColor:
             self._entry_icon.remove_css_class(color.value)
 
-        color = self._safe_entry.props.color
+        color = safe_entry.props.color
         self._entry_icon.add_css_class(color)
 
     @Gtk.Template.Callback()
